@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,14 +16,14 @@
 //!
 //! Module for establishing WebSocket connections with Splinter Services.
 //!
-//!``` no_run
+//!```
 //! use std::{thread::sleep, time};
 //! use splinter::events::{WsResponse, WebSocketClient, Reactor, ParseBytes};
 //!
 //! let reactor = Reactor::new();
 //!
 //! let mut ws = WebSocketClient::new(
-//!    "http://echo.websocket.org", |ctx, msg: Vec<u8>| {
+//!    "http://echo.websocket.org", "Bearer token", |ctx, msg: Vec<u8>| {
 //!    if let Ok(s) = String::from_utf8(msg.clone()) {
 //!         println!("Received {}", s);
 //!    } else {
@@ -74,7 +74,7 @@ use tokio::prelude::*;
 use crate::events::{Igniter, ParseError, WebSocketError};
 
 type OnErrorHandle<T> =
-    dyn Fn(&WebSocketError, Context<T>) -> Result<(), WebSocketError> + Send + Sync + 'static;
+    dyn Fn(WebSocketError, Context<T>) -> Result<(), WebSocketError> + Send + Sync + 'static;
 
 const MAX_FRAME_SIZE: usize = 10_000_000;
 const DEFAULT_RECONNECT: bool = false;
@@ -135,6 +135,7 @@ enum WebSocketClientCmd {
 /// WebSocket client. Configures Websocket connection and produces `Listen` future.
 pub struct WebSocketClient<T: ParseBytes<T> + 'static = Vec<u8>> {
     url: String,
+    authorization: String,
     on_message: Arc<dyn Fn(Context<T>, T) -> WsResponse + Send + Sync + 'static>,
     on_open: Option<Arc<dyn Fn(Context<T>) -> WsResponse + Send + Sync + 'static>>,
     on_error: Option<Arc<OnErrorHandle<T>>>,
@@ -149,6 +150,7 @@ impl<T: ParseBytes<T> + 'static> Clone for WebSocketClient<T> {
     fn clone(&self) -> Self {
         WebSocketClient {
             url: self.url.clone(),
+            authorization: self.authorization.clone(),
             on_message: self.on_message.clone(),
             on_open: self.on_open.clone(),
             on_error: self.on_error.clone(),
@@ -162,12 +164,13 @@ impl<T: ParseBytes<T> + 'static> Clone for WebSocketClient<T> {
 }
 
 impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
-    pub fn new<F>(url: &str, on_message: F) -> Self
+    pub fn new<F>(url: &str, authorization: &str, on_message: F) -> Self
     where
         F: Fn(Context<T>, T) -> WsResponse + Send + Sync + 'static,
     {
         Self {
             url: url.to_string(),
+            authorization: authorization.to_string(),
             on_message: Arc::new(on_message),
             on_open: None,
             on_error: None,
@@ -183,6 +186,10 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
         self.url.clone()
     }
 
+    pub fn authorization(&self) -> String {
+        self.authorization.clone()
+    }
+
     pub fn set_reconnect(&mut self, reconnect: bool) {
         self.reconnect = reconnect
     }
@@ -199,6 +206,10 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
         self.url = url.to_string();
     }
 
+    pub fn set_authorization(&mut self, authorization: &str) {
+        self.authorization = authorization.to_string();
+    }
+
     pub fn header(&mut self, header: &str, value: String) {
         self.additional_headers.insert(header.into(), value);
     }
@@ -230,7 +241,7 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
     /// Websocket or to reestablish the connection if appropriate.
     pub fn on_error<F>(&mut self, on_error: F)
     where
-        F: Fn(&WebSocketError, Context<T>) -> Result<(), WebSocketError> + Send + Sync + 'static,
+        F: Fn(WebSocketError, Context<T>) -> Result<(), WebSocketError> + Send + Sync + 'static,
     {
         self.on_error = Some(Arc::new(on_error));
     }
@@ -262,6 +273,10 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
             .on_error
             .clone()
             .unwrap_or_else(|| Arc::new(|_, _| Ok(())));
+        let on_stream_error = self
+            .on_error
+            .clone()
+            .unwrap_or_else(|| Arc::new(|_, _| Ok(())));
 
         let mut context_timeout = context.clone();
         let timeout = self.timeout;
@@ -273,7 +288,9 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
         debug!("starting: {}", url);
 
         let mut builder = Request::builder();
-        let mut request_builder = builder.uri(url);
+        let mut request_builder = builder
+            .uri(url)
+            .header("Authorization", &self.authorization);
 
         for (header, value) in self.additional_headers.iter() {
             request_builder = request_builder.header(header, value);
@@ -294,7 +311,7 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
                     if res.status() != StatusCode::SWITCHING_PROTOCOLS {
                         error!("The server didn't upgrade: {}", res.status());
                         if let Err(err) = on_error(
-                            &WebSocketError::ConnectError(format!(
+                            WebSocketError::ConnectError(format!(
                             "Received status code {:?} while attempting to establish a connection"
                         , res.status())),
                             connection_failed_context,
@@ -446,7 +463,13 @@ impl<T: ParseBytes<T> + 'static> WebSocketClient<T> {
                                 } else {
                                     match status {
                                         ConnectionStatus::Open => future::ok(true),
-                                        ConnectionStatus::UnexpectedClose(_original_error) => {
+                                        ConnectionStatus::UnexpectedClose(original_error) => {
+                                            if let Err(err) =
+                                                on_stream_error(original_error, context.clone())
+                                            {
+                                                error!("Failed to call on_error: {}", err);
+                                            }
+
                                             if let Err(err) = context.try_reconnect() {
                                                 error!("Context returned an error  {}", err);
                                             }
@@ -597,7 +620,7 @@ impl<T: ParseBytes<T> + 'static> Context<T> {
 
             self.reset_wait();
             self.reset_reconnect_count();
-            on_error(&error_message, self.clone())
+            on_error(error_message, self.clone())
         }
     }
 
