@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,9 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+mod builder;
 mod error;
 #[cfg(feature = "rest-api")]
 mod rest_api;
+mod runnable;
 
 use std::collections::HashMap;
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -24,10 +26,10 @@ use std::time::Duration;
 
 use crossbeam_channel::{Receiver, Sender};
 use protobuf::Message;
-use uuid::Uuid;
 
 use crate::channel;
-use crate::mesh::{Envelope, Mesh, RecvTimeoutError as MeshRecvTimeoutError};
+use crate::error::InternalError;
+use crate::mesh::{Envelope, Mesh, RecvTimeoutError as MeshRecvTimeoutError, SendError};
 use crate::network::reply::InboundRouter;
 use crate::protos::circuit::{
     AdminDirectMessage, CircuitDirectMessage, CircuitError, CircuitMessage, CircuitMessageType,
@@ -35,15 +37,17 @@ use crate::protos::circuit::{
 };
 use crate::protos::network::{NetworkMessage, NetworkMessageType};
 use crate::service::{
-    FactoryCreateError, Service, ServiceFactory, ServiceMessageContext,
-    StandardServiceNetworkRegistry,
+    Service, ServiceFactory, ServiceMessageContext, StandardServiceNetworkRegistry,
 };
+use crate::threading::lifecycle::ShutdownHandle;
 use crate::transport::Connection;
 
+pub use self::builder::ServiceOrchestratorBuilder;
 pub use self::error::{
-    InitializeServiceError, ListServicesError, NewOrchestratorError, OrchestratorError,
-    ShutdownServiceError,
+    AddServiceError, InitializeServiceError, ListServicesError, NewOrchestratorError,
+    OrchestratorError, ShutdownServiceError,
 };
+pub use self::runnable::RunnableServiceOrchestrator;
 
 // Recv timeout in secs
 const TIMEOUT_SEC: u64 = 2;
@@ -56,38 +60,19 @@ pub struct ServiceDefinition {
     pub service_type: String,
 }
 
-/// A service that may be orchestratable.
-///
-/// This service has several stronger requirements, mainly required moving and sharing a service
-/// instance among threads.
-pub trait OrchestratableService: Service {
-    fn clone_box(&self) -> Box<dyn OrchestratableService>;
-
-    fn as_service(&self) -> &dyn Service;
-}
-
-impl Clone for Box<dyn OrchestratableService> {
-    fn clone(&self) -> Self {
-        self.clone_box()
+impl std::fmt::Display for ServiceDefinition {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        write!(
+            f,
+            "{}::{} ({})",
+            self.circuit, self.service_id, self.service_type
+        )
     }
 }
 
-/// A service factory that produces orchestratable services.
-pub trait OrchestratableServiceFactory: ServiceFactory {
-    /// Create a Service instance with the given ID, of the given type, the given circuit_id,
-    /// with the given arguments.
-    fn create_orchestratable_service(
-        &self,
-        service_id: String,
-        service_type: &str,
-        circuit_id: &str,
-        args: HashMap<String, String>,
-    ) -> Result<Box<dyn OrchestratableService>, FactoryCreateError>;
-}
-
 /// Stores a service and other structures that are used to manage it
 struct ManagedService {
-    pub service: Box<dyn OrchestratableService>,
+    pub service: Box<dyn Service>,
     pub registry: StandardServiceNetworkRegistry,
 }
 
@@ -96,122 +81,58 @@ pub struct ServiceOrchestrator {
     /// A (ServiceDefinition, ManagedService) map
     services: Arc<Mutex<HashMap<ServiceDefinition, ManagedService>>>,
     /// Factories used to create new services.
-    service_factories: Vec<Box<dyn OrchestratableServiceFactory>>,
+    service_factories: Vec<Box<dyn ServiceFactory>>,
     supported_service_types: Vec<String>,
     /// `network_sender` and `inbound_router` are used to create services' senders.
     network_sender: Sender<Vec<u8>>,
     inbound_router: InboundRouter<CircuitMessageType>,
+    /// A (ServiceDefinition, ManagedService) map of services that have been stopped, but yet to
+    /// be completely destroyed
+    stopped_services: Arc<Mutex<HashMap<ServiceDefinition, Box<dyn Service>>>>,
+
     /// `running` and `join_handles` are used to shutdown the orchestrator's background threads
     running: Arc<AtomicBool>,
+    join_handles: Option<JoinHandles<Result<(), OrchestratorError>>>,
 }
 
 impl ServiceOrchestrator {
     /// Create a new `ServiceOrchestrator`. This starts up 3 threads for relaying messages to and
     /// from services. Returns the `ServiceOrchestrator` and the threads `JoinHandles`
+    #[deprecated(
+        since = "0.5.1",
+        note = "please use `ServiceOrchestratorBuilder` instead"
+    )]
     pub fn new(
-        service_factories: Vec<Box<dyn OrchestratableServiceFactory>>,
+        service_factories: Vec<Box<dyn ServiceFactory>>,
         connection: Box<dyn Connection>,
         incoming_capacity: usize,
         outgoing_capacity: usize,
         channel_capacity: usize,
-    ) -> Result<(Self, JoinHandles<Result<(), OrchestratorError>>), NewOrchestratorError> {
-        let services = Arc::new(Mutex::new(HashMap::new()));
-        let mesh = Mesh::new(incoming_capacity, outgoing_capacity);
-        let mesh_id = format!("{}", Uuid::new_v4());
-        mesh.add(connection, mesh_id.to_string())
-            .map_err(|err| NewOrchestratorError(Box::new(err)))?;
-        let (network_sender, network_receiver) = crossbeam_channel::bounded(channel_capacity);
-        let (inbound_sender, inbound_receiver) = crossbeam_channel::bounded(channel_capacity);
-        let inbound_router = InboundRouter::new(Box::new(inbound_sender));
-        let running = Arc::new(AtomicBool::new(true));
+    ) -> Result<Self, NewOrchestratorError> {
+        let mut builder = builder::ServiceOrchestratorBuilder::new()
+            .with_connection(connection)
+            .with_incoming_capacity(incoming_capacity)
+            .with_outgoing_capacity(outgoing_capacity)
+            .with_channel_capacity(channel_capacity);
 
-        debug!("Orchestrator authorized");
-
-        // Start thread that handles incoming messages from a splinter node.
-        let incoming_mesh = mesh.clone();
-        let incoming_running = running.clone();
-        let incoming_router = inbound_router.clone();
-        let incoming_join_handle = thread::Builder::new()
-            .name("Orchestrator Incoming".into())
-            .spawn(move || {
-                if let Err(err) =
-                    run_incoming_loop(incoming_mesh, incoming_running, incoming_router)
-                {
-                    error!(
-                        "Terminating orchestrator incoming thread due to error: {}",
-                        err
-                    );
-                    Err(err)
-                } else {
-                    Ok(())
-                }
-            })
-            .map_err(|err| NewOrchestratorError(Box::new(err)))?;
-
-        // Start thread that handles messages that do not have a matching correlation id.
-        let inbound_services = services.clone();
-        let inbound_running = running.clone();
-        let inbound_join_handle = thread::Builder::new()
-            .name("Orchestrator Inbound".into())
-            .spawn(move || {
-                if let Err(err) =
-                    run_inbound_loop(inbound_services, inbound_receiver, inbound_running)
-                {
-                    error!(
-                        "Terminating orchestrator inbound thread due to error: {}",
-                        err
-                    );
-                    Err(err)
-                } else {
-                    Ok(())
-                }
-            })
-            .map_err(|err| NewOrchestratorError(Box::new(err)))?;
-
-        // Start thread that handles outgoing messages that need to be sent to the splinter node.
-        let outgoing_running = running.clone();
-        let outgoing_join_handle = thread::Builder::new()
-            .name("Orchestrator Outgoing".into())
-            .spawn(move || {
-                if let Err(err) =
-                    run_outgoing_loop(mesh, outgoing_running, network_receiver, mesh_id)
-                {
-                    error!(
-                        "Terminating orchestrator outgoing thread due to error: {}",
-                        err
-                    );
-                    Err(err)
-                } else {
-                    Ok(())
-                }
-            })
-            .map_err(|err| NewOrchestratorError(Box::new(err)))?;
-
-        let supported_service_types_vec = service_factories
-            .iter()
-            .map(|factory| factory.available_service_types().to_vec())
-            .collect::<Vec<Vec<String>>>();
-
-        let mut supported_service_types = vec![];
-        for mut service_types in supported_service_types_vec {
-            supported_service_types.append(&mut service_types);
+        for service_factory in service_factories.into_iter() {
+            builder = builder.with_service_factory(service_factory);
         }
 
-        Ok((
-            Self {
-                services,
-                service_factories,
-                supported_service_types,
-                network_sender,
-                inbound_router,
-                running,
-            },
-            JoinHandles::new(vec![
-                incoming_join_handle,
-                inbound_join_handle,
-                outgoing_join_handle,
-            ]),
-        ))
+        builder
+            .build()
+            .map_err(|e| NewOrchestratorError(Box::new(e)))?
+            .run()
+            .map_err(|e| NewOrchestratorError(Box::new(e)))
+    }
+
+    pub fn take_shutdown_handle(&mut self) -> Option<ServiceOrchestratorShutdownHandle> {
+        let join_handles = self.join_handles.take()?;
+        Some(ServiceOrchestratorShutdownHandle {
+            services: Arc::clone(&self.services),
+            join_handles: Some(join_handles),
+            running: Arc::clone(&self.running),
+        })
     }
 
     /// Initialize (create and start) a service according to the specified definition. The
@@ -233,7 +154,7 @@ impl ServiceOrchestrator {
             .ok_or(InitializeServiceError::UnknownType)?;
 
         // Create the service.
-        let mut service = factory.create_orchestratable_service(
+        let mut service = factory.create(
             service_definition.service_id.clone(),
             service_definition.service_type.as_str(),
             service_definition.circuit.as_str(),
@@ -260,8 +181,8 @@ impl ServiceOrchestrator {
         Ok(())
     }
 
-    /// Shut down (stop and destroy) the specified service.
-    pub fn shutdown_service(
+    /// Stop the specified service.
+    pub fn stop_service(
         &self,
         service_definition: &ServiceDefinition,
     ) -> Result<(), ShutdownServiceError> {
@@ -278,13 +199,34 @@ impl ServiceOrchestrator {
         service.stop(&registry).map_err(|err| {
             ShutdownServiceError::ShutdownFailed((service_definition.clone(), Box::new(err)))
         })?;
-        service.destroy().map_err(|err| {
-            ShutdownServiceError::ShutdownFailed((service_definition.clone(), Box::new(err)))
-        })?;
+
+        self.stopped_services
+            .lock()
+            .map_err(|_| ShutdownServiceError::LockPoisoned)?
+            .insert(service_definition.clone(), service);
 
         Ok(())
     }
 
+    /// Purge the specified service state, based on its service implementation.
+    pub fn purge_service(
+        &self,
+        service_definition: &ServiceDefinition,
+    ) -> Result<(), InternalError> {
+        if let Some(mut service) = self
+            .stopped_services
+            .lock()
+            .map_err(|_| {
+                InternalError::with_message("Orchestrator stopped service lock was poisoned".into())
+            })?
+            .remove(service_definition)
+        {
+            service.purge()
+        } else {
+            Ok(())
+        }
+    }
+
     /// Shut down (stop and destroy) all services managed by this `ServiceOrchestrator` and single
     /// the `ServiceOrchestrator` to shutdown
     pub fn shutdown_all_services(&self) -> Result<(), ShutdownServiceError> {
@@ -335,6 +277,41 @@ impl ServiceOrchestrator {
             .collect())
     }
 
+    /// Create a service that has previously been stopped according to the specified definition.
+    /// The arguments provided must match those required to create the service.
+    pub fn add_stopped_service(
+        &self,
+        service_definition: ServiceDefinition,
+        args: HashMap<String, String>,
+    ) -> Result<(), AddServiceError> {
+        // Get the factory that can create this service.
+        let factory = self
+            .service_factories
+            .iter()
+            .find(|factory| {
+                factory
+                    .available_service_types()
+                    .contains(&service_definition.service_type)
+            })
+            .ok_or(AddServiceError::UnknownType)?;
+
+        // Create the previously stopped service.
+        let service = factory.create(
+            service_definition.service_id.clone(),
+            service_definition.service_type.as_str(),
+            service_definition.circuit.as_str(),
+            args,
+        )?;
+
+        // Save the service to `stopped_services`.
+        self.stopped_services
+            .lock()
+            .map_err(|_| AddServiceError::LockPoisoned)?
+            .insert(service_definition, service);
+
+        Ok(())
+    }
+
     pub fn supported_service_types(&self) -> &[String] {
         &self.supported_service_types
     }
@@ -360,7 +337,61 @@ impl<T> JoinHandles<T> {
     }
 }
 
-pub fn run_incoming_loop(
+pub struct ServiceOrchestratorShutdownHandle {
+    services: Arc<Mutex<HashMap<ServiceDefinition, ManagedService>>>,
+    join_handles: Option<JoinHandles<Result<(), OrchestratorError>>>,
+    running: Arc<AtomicBool>,
+}
+
+impl ShutdownHandle for ServiceOrchestratorShutdownHandle {
+    fn signal_shutdown(&mut self) {
+        match self.services.lock() {
+            Ok(mut services) => {
+                for (service_definition, managed_service) in services.drain() {
+                    let ManagedService {
+                        mut service,
+                        registry,
+                    } = managed_service;
+                    if let Err(err) = service.stop(&registry) {
+                        error!("Unable to stop service {}: {}", service_definition, err);
+                    }
+                    if let Err(err) = service.destroy() {
+                        error!("Unable to destroy service {}: {}", service_definition, err);
+                    }
+                }
+            }
+            Err(_) => {
+                error!("Service orchestrator service lock was poisoned; unable to cleanly shutdown")
+            }
+        }
+        self.running.store(false, Ordering::SeqCst);
+    }
+
+    fn wait_for_shutdown(mut self) -> Result<(), InternalError> {
+        if let Some(join_handles) = self.join_handles.take() {
+            match join_handles.join_all() {
+                Ok(results) => {
+                    results
+                        .into_iter()
+                        .filter(Result::is_err)
+                        .map(Result::unwrap_err)
+                        .for_each(|err| {
+                            error!("{}", err);
+                        });
+                }
+                Err(_) => {
+                    return Err(crate::error::InternalError::with_message(
+                        "Unable to join service processor threads".into(),
+                    ));
+                }
+            }
+        }
+
+        Ok(())
+    }
+}
+
+fn run_incoming_loop(
     incoming_mesh: Mesh,
     incoming_running: Arc<AtomicBool>,
     mut inbound_router: InboundRouter<CircuitMessageType>,
@@ -392,7 +423,7 @@ pub fn run_incoming_loop(
         // the message, otherwise it will be sent to the inbound thread
         match msg.get_message_type() {
             NetworkMessageType::CIRCUIT => {
-                let mut circuit_msg: CircuitMessage = Message::parse_from_bytes(msg.get_payload())
+                let mut circuit_msg: CircuitMessage = Message::parse_from_bytes(&msg.get_payload())
                     .map_err(|err| OrchestratorError::Internal(Box::new(err)))?;
 
                 match circuit_msg.get_message_type() {
@@ -589,9 +620,12 @@ fn run_outgoing_loop(
         };
 
         // Send message to splinter node
-        outgoing_mesh
-            .send(Envelope::new(mesh_id.to_string(), message_bytes))
-            .map_err(|err| OrchestratorError::Internal(Box::new(err)))?;
+        match outgoing_mesh.send(Envelope::new(mesh_id.to_string(), message_bytes)) {
+            Ok(()) => (),
+            // drop message if it cannot be sent because the queue is full
+            Err(SendError::Full(_)) => error!("Unable to send outgoing message, queue full"),
+            Err(err) => return Err(OrchestratorError::Internal(Box::new(err))),
+        }
     }
     Ok(())
 }
