@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -143,14 +143,14 @@ impl PeerManagerConnector {
         match self.sender.send(message) {
             Ok(()) => (),
             Err(_) => {
-                return Err(PeerListError::Internal(
+                return Err(PeerListError::InternalError(
                     "Unable to send message to PeerManager, receiver dropped".to_string(),
                 ))
             }
         };
 
         recv.recv()
-            .map_err(|err| PeerListError::Receive(format!("{:?}", err)))?
+            .map_err(|err| PeerListError::ReceiveError(format!("{:?}", err)))?
     }
 
     /// Requests the list of unreferenced peers.
@@ -166,14 +166,14 @@ impl PeerManagerConnector {
         match self.sender.send(message) {
             Ok(()) => (),
             Err(_) => {
-                return Err(PeerListError::Internal(
+                return Err(PeerListError::InternalError(
                     "Unable to send message to PeerManager, receiver dropped".to_string(),
                 ))
             }
         };
 
         recv.recv()
-            .map_err(|err| PeerListError::Receive(format!("{:?}", err)))?
+            .map_err(|err| PeerListError::ReceiveError(format!("{:?}", err)))?
     }
 
     /// Requests the map of currently connected peers to connection IDs
@@ -200,6 +200,7 @@ impl PeerManagerConnector {
     ///
     /// Returns a `PeerNotificationIter` that can be used to receive notifications about connected
     /// and disconnected peers
+    #[deprecated(since = "0.5.1", note = "please use `subscribe_sender` instead")]
     pub fn subscribe(&self) -> Result<PeerNotificationIter, PeerManagerError> {
         let (send, recv) = channel();
         match self.sender.send(PeerManagerMessage::Subscribe(send)) {
@@ -344,14 +345,14 @@ impl PeerRemover {
         match self.sender.send(message) {
             Ok(()) => (),
             Err(_) => {
-                return Err(PeerRefRemoveError::Internal(
+                return Err(PeerRefRemoveError::InternalError(
                     "Unable to send message to PeerManager, receiver dropped".to_string(),
                 ))
             }
         };
 
         recv.recv()
-            .map_err(|err| PeerRefRemoveError::Receive(format!("{:?}", err)))?
+            .map_err(|err| PeerRefRemoveError::ReceiveError(format!("{:?}", err)))?
     }
 
     /// Sends a request to the `PeerManager` to remove a peer by its endpoint.
@@ -371,14 +372,14 @@ impl PeerRemover {
         match self.sender.send(message) {
             Ok(()) => (),
             Err(_) => {
-                return Err(PeerRefRemoveError::Internal(
+                return Err(PeerRefRemoveError::InternalError(
                     "Unable to send message to PeerManager, receiver dropped".to_string(),
                 ))
             }
         };
 
         recv.recv()
-            .map_err(|err| PeerRefRemoveError::Receive(format!("{:?}", err)))?
+            .map_err(|err| PeerRefRemoveError::ReceiveError(format!("{:?}", err)))?
     }
 }
 
