@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,35 +18,54 @@
 //! * `POST /registry/nodes` for adding a node to the registry
 
 use std::collections::HashMap;
+use std::convert::TryFrom;
 
 use crate::actix_web::{error::BlockingError, web, Error, HttpRequest, HttpResponse};
+use crate::error::InvalidStateError;
 use crate::futures::{future::IntoFuture, stream::Stream, Future};
 use crate::protocol;
+use crate::registry::rest_api::error::RegistryRestApiError;
+#[cfg(feature = "authorization")]
+use crate::registry::rest_api::{REGISTRY_READ_PERMISSION, REGISTRY_WRITE_PERMISSION};
 use crate::registry::{
-    rest_api::resources::nodes::{ListNodesResponse, NodeResponse},
-    InvalidNodeError, MetadataPredicate, Node, RegistryError, RegistryReader, RegistryWriter,
-    RwRegistry,
+    rest_api::resources::nodes::{ListNodesResponse, NewNode, NodeResponse},
+    MetadataPredicate, Node, RegistryReader, RegistryWriter, RwRegistry,
 };
 use crate::rest_api::{
+    actix_web_1::{Method, ProtocolVersionRangeGuard, Resource},
     paging::{get_response_paging_info, DEFAULT_LIMIT, DEFAULT_OFFSET},
-    percent_encode_filter_query, ErrorResponse, Method, ProtocolVersionRangeGuard, Resource,
+    percent_encode_filter_query, ErrorResponse,
 };
 
 type Filter = HashMap<String, (String, String)>;
 
 pub fn make_nodes_resource(registry: Box<dyn RwRegistry>) -> Resource {
     let registry1 = registry.clone();
-    Resource::build("/registry/nodes")
-        .add_request_guard(ProtocolVersionRangeGuard::new(
+    let resource =
+        Resource::build("/registry/nodes").add_request_guard(ProtocolVersionRangeGuard::new(
             protocol::REGISTRY_LIST_NODES_MIN,
             protocol::REGISTRY_PROTOCOL_VERSION,
-        ))
-        .add_method(Method::Get, move |r, _| {
-            list_nodes(r, web::Data::new(registry.clone_box_as_reader()))
-        })
-        .add_method(Method::Post, move |_, p| {
-            add_node(p, web::Data::new(registry1.clone()))
-        })
+        ));
+    #[cfg(feature = "authorization")]
+    {
+        resource
+            .add_method(Method::Get, REGISTRY_READ_PERMISSION, move |r, _| {
+                list_nodes(r, web::Data::new(registry.clone_box_as_reader()))
+            })
+            .add_method(Method::Post, REGISTRY_WRITE_PERMISSION, move |_, p| {
+                add_node(p, web::Data::new(registry1.clone()))
+            })
+    }
+    #[cfg(not(feature = "authorization"))]
+    {
+        resource
+            .add_method(Method::Get, move |r, _| {
+                list_nodes(r, web::Data::new(registry.clone_box_as_reader()))
+            })
+            .add_method(Method::Post, move |_, p| {
+                add_node(p, web::Data::new(registry1.clone()))
+            })
+    }
 }
 
 fn list_nodes(
@@ -151,7 +170,9 @@ fn query_list_nodes(
     limit: Option<usize>,
 ) -> impl Future<Item = HttpResponse, Error = Error> {
     web::block(move || {
-        let nodes = registry.list_nodes(&filters)?;
+        let nodes = registry
+            .list_nodes(&filters)
+            .map_err(RegistryRestApiError::from)?;
         let offset_value = offset.unwrap_or(0);
         let total = nodes.len();
         let limit_value = limit.unwrap_or_else(|| total as usize);
@@ -163,18 +184,20 @@ fn query_list_nodes(
 
         Ok((nodes, link, limit, offset, total as usize))
     })
-    .then(|res: Result<_, BlockingError<RegistryError>>| match res {
-        Ok((nodes, link, limit, offset, total_count)) => {
-            Ok(HttpResponse::Ok().json(ListNodesResponse {
-                data: nodes.iter().map(NodeResponse::from).collect(),
-                paging: get_response_paging_info(limit, offset, &link, total_count),
-            }))
-        }
-        Err(err) => {
-            error!("Unable to list nodes: {}", err);
-            Ok(HttpResponse::InternalServerError().json(ErrorResponse::internal_error()))
-        }
-    })
+    .then(
+        |res: Result<_, BlockingError<RegistryRestApiError>>| match res {
+            Ok((nodes, link, limit, offset, total_count)) => {
+                Ok(HttpResponse::Ok().json(ListNodesResponse {
+                    data: nodes.iter().map(NodeResponse::from).collect(),
+                    paging: get_response_paging_info(limit, offset, &link, total_count),
+                }))
+            }
+            Err(err) => {
+                error!("Unable to list nodes: {}", err);
+                Ok(HttpResponse::InternalServerError().json(ErrorResponse::internal_error()))
+            }
+        },
+    )
 }
 
 fn to_predicates(filters: Option<Filter>) -> Result<Vec<MetadataPredicate>, String> {
@@ -207,25 +230,29 @@ fn add_node(
                 Ok::<_, Error>(body)
             })
             .into_future()
-            .and_then(move |body| match serde_json::from_slice::<Node>(&body) {
+            .and_then(move |body| match serde_json::from_slice::<NewNode>(&body) {
                 Ok(node) => Box::new(
                     web::block(move || {
-                        if registry.has_node(&node.identity)? {
-                            Err(RegistryError::InvalidNode(
-                                InvalidNodeError::DuplicateIdentity(node.identity),
-                            ))
-                        } else {
-                            registry.insert_node(node)
-                        }
+                        let new_node = Node::try_from(node).map_err(|err| {
+                            RegistryRestApiError::InvalidStateError(
+                                InvalidStateError::with_message(format!(
+                                    "Failed to add node, node is invalid: {}",
+                                    err
+                                )),
+                            )
+                        })?;
+                        registry
+                            .add_node(new_node)
+                            .map_err(RegistryRestApiError::from)
                     })
                     .then(|res| {
                         Ok(match res {
                             Ok(_) => HttpResponse::Ok().finish(),
-                            Err(BlockingError::Error(RegistryError::InvalidNode(err))) => {
-                                HttpResponse::BadRequest().json(ErrorResponse::bad_request(
-                                    &format!("Invalid node: {}", err),
-                                ))
-                            }
+                            Err(BlockingError::Error(RegistryRestApiError::InvalidStateError(
+                                err,
+                            ))) => HttpResponse::BadRequest().json(ErrorResponse::bad_request(
+                                &format!("Invalid node: {}", err),
+                            )),
                             Err(err) => {
                                 error!("Unable to add node: {}", err);
                                 HttpResponse::InternalServerError()
@@ -256,9 +283,11 @@ mod tests {
     use reqwest::{blocking::Client, StatusCode, Url};
     use serde_json::{to_value, Value as JsonValue};
 
-    use crate::registry::NodeIter;
+    use crate::error::InvalidStateError;
+    use crate::registry::{error::RegistryError, NodeIter};
     use crate::rest_api::{
-        paging::Paging, RestApiBuilder, RestApiServerError, RestApiShutdownHandle,
+        actix_web_1::{RestApiBuilder, RestApiShutdownHandle},
+        paging::Paging,
     };
 
     #[test]
@@ -427,27 +456,12 @@ mod tests {
                 "SplinterProtocolVersion",
                 protocol::REGISTRY_PROTOCOL_VERSION,
             )
-            .json(&get_node_1())
+            .json(&get_new_node_1())
             .send()
             .expect("Failed to perform request");
 
         assert_eq!(resp.status(), StatusCode::OK);
 
-        // Verify a duplicate node gets a BAD_REQUEST response
-        let url = Url::parse(&format!("http://{}/registry/nodes", bind_url))
-            .expect("Failed to parse URL");
-        let resp = Client::new()
-            .post(url)
-            .header(
-                "SplinterProtocolVersion",
-                protocol::REGISTRY_PROTOCOL_VERSION,
-            )
-            .json(&get_node_1())
-            .send()
-            .expect("Failed to perform request");
-
-        assert_eq!(resp.status(), StatusCode::BAD_REQUEST);
-
         shutdown_handle
             .shutdown()
             .expect("Unable to shutdown rest api");
@@ -457,24 +471,24 @@ mod tests {
     fn run_rest_api_on_open_port(
         resources: Vec<Resource>,
     ) -> (RestApiShutdownHandle, std::thread::JoinHandle<()>, String) {
-        (10000..20000)
-            .find_map(|port| {
-                let bind_url = format!("127.0.0.1:{}", port);
-                let result = RestApiBuilder::new()
-                    .with_bind(&bind_url)
-                    .add_resources(resources.clone())
-                    .build()
-                    .expect("Failed to build REST API")
-                    .run();
-                match result {
-                    Ok((shutdown_handle, join_handle)) => {
-                        Some((shutdown_handle, join_handle, bind_url))
-                    }
-                    Err(RestApiServerError::BindError(_)) => None,
-                    Err(err) => panic!("Failed to run REST API: {}", err),
-                }
-            })
-            .expect("No port available")
+        #[cfg(not(feature = "https-bind"))]
+        let bind = "127.0.0.1:0";
+        #[cfg(feature = "https-bind")]
+        let bind = crate::rest_api::BindConfig::Http("127.0.0.1:0".into());
+
+        let result = RestApiBuilder::new()
+            .with_bind(bind)
+            .add_resources(resources.clone())
+            .build_insecure()
+            .expect("Failed to build REST API")
+            .run_insecure();
+        match result {
+            Ok((shutdown_handle, join_handle)) => {
+                let port = shutdown_handle.port_numbers()[0];
+                (shutdown_handle, join_handle, format!("127.0.0.1:{}", port))
+            }
+            Err(err) => panic!("Failed to run REST API: {}", err),
+        }
     }
 
     fn create_test_paging_response(
@@ -515,6 +529,19 @@ mod tests {
             .expect("Failed to build node1")
     }
 
+    fn get_new_node_1() -> NewNode {
+        let mut metadata = HashMap::new();
+        metadata.insert("company".into(), "Bitwise IO".into());
+
+        NewNode {
+            identity: "Node-123".into(),
+            endpoints: vec!["12.0.0.123:8431".into()],
+            display_name: "Bitwise IO - Node 1".into(),
+            keys: vec!["0123".into()],
+            metadata,
+        }
+    }
+
     fn get_node_2() -> Node {
         Node::builder("Node-456")
             .with_endpoint("13.0.0.123:8434")
@@ -560,7 +587,7 @@ mod tests {
             self.list_nodes(predicates).map(|iter| iter.count() as u32)
         }
 
-        fn fetch_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
+        fn get_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
             Ok(self
                 .nodes
                 .lock()
@@ -571,7 +598,7 @@ mod tests {
     }
 
     impl RegistryWriter for MemRegistry {
-        fn insert_node(&self, node: Node) -> Result<(), RegistryError> {
+        fn add_node(&self, node: Node) -> Result<(), RegistryError> {
             self.nodes
                 .lock()
                 .expect("mem registry lock was poisoned")
@@ -579,6 +606,22 @@ mod tests {
             Ok(())
         }
 
+        fn update_node(&self, node: Node) -> Result<(), RegistryError> {
+            let mut inner = self.nodes.lock().expect("mem registry lock was poisoned");
+
+            if inner.contains_key(&node.identity) {
+                inner.insert(node.identity.clone(), node);
+                Ok(())
+            } else {
+                Err(RegistryError::InvalidStateError(
+                    InvalidStateError::with_message(format!(
+                        "Node does not exist in the registry: {}",
+                        node.identity
+                    )),
+                ))
+            }
+        }
+
         fn delete_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
             Ok(self
                 .nodes
