@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 
 //! Dispatch handlers for service component messages.
 
-use crate::circuit::service::ServiceId;
+use crate::circuit::routing::ServiceId;
 use crate::network::dispatch::{
     ConnectionId, DispatchError, DispatchMessageSender, Handler, MessageContext, MessageSender,
 };
@@ -149,7 +149,7 @@ impl std::fmt::Display for ServiceAddInstanceError {
             ServiceAddInstanceError::InternalError {
                 context,
                 source: None,
-            } => f.write_str(context),
+            } => f.write_str(&context),
         }
     }
 }
@@ -199,7 +199,7 @@ impl std::fmt::Display for ServiceRemoveInstanceError {
             ServiceRemoveInstanceError::InternalError {
                 context,
                 source: None,
-            } => f.write_str(context),
+            } => f.write_str(&context),
         }
     }
 }
