@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -288,13 +288,13 @@ macro_rules! agent_msg {
         }
     };
 
-    ($sender:expr, $msg_type:ident { $($field:ident: $value:expr,)* }) => {
+    ($sender:expr, $msg_type:ident { $($fields:tt)* }) => {
         {
             let (tx, rx) = mpsc::channel();
             agent_msg!(@do_send $sender, rx,
                 AgentMessage::$msg_type {
                     reply_sender: tx,
-                    $($field: $value)*
+                    $($fields)*
                 })
         }
     };
@@ -366,14 +366,8 @@ impl ServiceConnector {
         )
     }
 
-    #[allow(clippy::redundant_field_names)]
     pub fn unsubscribe(&self, subscriber_id: SubscriberId) -> Result<(), ServiceConnectionError> {
-        agent_msg!(
-            self.sender,
-            Unsubscribe {
-                subscriber_id: subscriber_id,
-            }
-        )
+        agent_msg!(self.sender, Unsubscribe { subscriber_id })
     }
 }
 
@@ -426,7 +420,7 @@ impl ServiceConnectionAgent {
         loop {
             match self.receiver.recv() {
                 Ok(AgentMessage::ConnectionNotification(notification)) => {
-                    self.handle_notification(notification)?;
+                    self.handle_notification(notification);
                 }
                 Ok(AgentMessage::ListServices { reply_sender }) => {
                     self.list_services(reply_sender)?;
@@ -514,10 +508,7 @@ impl ServiceConnectionAgent {
         )
     }
 
-    fn handle_notification(
-        &mut self,
-        notification: ConnectionManagerNotification,
-    ) -> Result<(), ServiceConnectionAgentError> {
+    fn handle_notification(&mut self, notification: ConnectionManagerNotification) {
         match notification {
             ConnectionManagerNotification::InboundConnection {
                 endpoint,
@@ -576,8 +567,6 @@ impl ServiceConnectionAgent {
                 }
             }
         }
-
-        Ok(())
     }
 }
 
@@ -654,6 +643,7 @@ mod tests {
     use crate::network::connection_manager::{
         AuthorizationResult, Authorizer, AuthorizerCallback, AuthorizerError, ConnectionManager,
     };
+    use crate::threading::lifecycle::ShutdownHandle;
     use crate::transport::{inproc::InprocTransport, Connection, Transport};
 
     impl ServiceConnectionManager {
@@ -674,7 +664,7 @@ mod tests {
         let mut listener = transport.listen("inproc://test_service_connected").unwrap();
 
         let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("service-id")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -746,8 +736,9 @@ mod tests {
         jh.join().unwrap();
 
         service_conn_mgr.shutdown_and_wait();
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     struct NoopAuthorizer {
